#include <ESP8266WiFi.h>
#include <WiFiClient.h> 
#include <ESP8266WebServer.h>
#include <ESP8266HTTPClient.h>
#include <stdio.h>
#include <stdlib.h>
#include <hspi_slave.h>
#include <SPISlave.h>

/* Set these to your desired credentials. */
const char WiFiSSID[] = "BAT_NEWS_TRAVELS_FAST";
const char WiFiPSK[] = "cmbisgreat";

unsigned int gain;
int recordVal;
int fileDownload;
uint8_t buf[4096];

// set debug mode to true for serial output
int debugMode = 0;

enum Status {
    NO_STATUS = 0x10,
    START_REC = 0x11,
    STOP_REC = 0x12,
    CHANGE_GAIN = 0x13,
    FILE_DOWNLOAD = 0x14
} stat;

String picStatus;

#define RECORDING 2
#define DOWNLOADING 3

#define GET_UPDATE 0x20
#define MTU_SIZE 1406

ESP8266WebServer server(80);
WiFiClient client;

// handle file download
void download() {
  client = server.client();
  //client.setNoDelay(1);      // don't think this does anything
  client.print("HTTP/1.1 200 OK\r\n");
  client.print("Content-Disposition: attachment; filename=config.txt\r\n");
  client.print("Content-Type: application/octet-stream\r\n");
  client.print("Content-Length: 10485760\r\n");
  client.print("Connection: close\r\n");
  client.print("Access-Control-Allow-Origin: *\r\n");
  client.print("\r\n");
  int i;
  for (i = 0; i < 10485760/MTU_SIZE; i++) {
    Serial.println(millis());
    client.write((const char*)buf, MTU_SIZE);
  }
}

// build webpage and respond to input
void webpage() {
  server.send(200, "text/html", 
    "<html>"
    "<style>html"
      "{ font-family: Bookman; color:#674EA7; display: inline-block; margin: 0px auto; text-align: center;}"
     "</style>"
     "<body style='background-color:#D7D7D7'>"
        "<h1>"
          "BAT NEWS TRAVELS FAST!<br>"
          "Welcome to the Bat Ear Interface"
        "</h1>"
        "<p>"
         "Instructions:<br>"
         "To download the data file, go to URL 192.168.4.1/download<br>"
         "Change the Gain by making your desired selection in the box below. "
         "Start or stop the recording by making a selection on the radio buttons.<br>"
         "Note: only one change can be made at a time<br><br>"
         "Hit submit to make your dreams a reality!<br><br>"
         "Desired Amplifier Gain:"
        "</p>"
        "<form method=get name=LED>"
          "<input type='number' name='x'><br>"
          "<input type='radio' name='L' value='1'>START RECORDING<br>"
          "<input type='radio' name='L' value='0'>STOP RECORDING<br>"
          "<input type=submit value=submit>"
        "</form>"
      "</body>"
    "</html>");

  // CHANGE GAIN
  if(server.hasArg("x") && server.arg("x").length() > 0) { // TODO check that it's not longer than 31 characters
    gain = server.arg("x").toInt();
    stat = CHANGE_GAIN;
    SPISlave.setStatus(stat | (gain << 8));
    
    if (debugMode) {
      Serial.print("User entered:\t");
      Serial.println(server.arg("x"));
      Serial.print("The gain is now:\t");    
      Serial.println(gain);
      Serial.print("Current status:\t");
      Serial.println(stat);
    }
  }
     
  // CHANGE RECORDING STATUS
  else if(server.hasArg("L")) {    
    recordVal = server.arg("L").toInt();
    if (recordVal)
      stat = START_REC;
    else
      stat = STOP_REC;
    SPISlave.setStatus(stat);
    
    if (debugMode) {
      Serial.print("User entered recording status=:\t");
      Serial.println(server.arg("L"));
      Serial.print("The recording status is now:\t");
      Serial.println(recordVal);
      Serial.print("Current status:\t");
      Serial.println(stat);
    }
  }       

  //CHANGE FILE DOWNLOAD STATUS
  if(server.hasArg("F")) {
    fileDownload = server.arg("F").toInt();
    stat = FILE_DOWNLOAD;
    SPISlave.setStatus(FILE_DOWNLOAD);
    download();   // can we just call this here?

    if (debugMode) {
      Serial.print("User entered filedownload=:\t");
      Serial.println(server.arg("F"));
      Serial.print("The file download status is now:\t");
      Serial.println(fileDownload);
      Serial.print("Current status:\t");
      Serial.println(stat);
    }
  }
}

  
void setup() {
  Serial.begin(115200);
  Serial.println();
  Serial.println("Configuring access point...");
  Serial.println();
  
  WiFi.softAP(WiFiSSID, WiFiPSK);
  IPAddress myIP = WiFi.softAPIP();
  
  Serial.println("AP IP address: ");
  Serial.println(myIP);
  
  server.on("/",HTTP_GET, webpage);
  server.on("/",HTTP_POST,webpage);
  
  /* let's try this fucking idea */
  server.on("/download", HTTP_GET, download);
  server.on("/download", HTTP_POST, download);
  
  server.begin();
  
  Serial.println();
  Serial.println("HTTP server started");
  Serial.setDebugOutput(true);
  
  SPISlave.setStatus(NO_STATUS);

  // make a dummy buffer to download
  for (int i = 0; i < 4096; i++)
    buf[i] = 'x';

  // data has been received from the master. Beware that len is always 32
  // and the buffer is autofilled with zeroes if data is less than 32 bytes long
  // It's up to the user to implement protocol for handling data length
  SPISlave.onData([](uint8_t * data, size_t len) {
    if (data[0] == RECORDING)
      picStatus = "Recording";
    else if (data[0] == DOWNLOADING)
      picStatus = "Downloading";
    else
      picStatus = "Waiting...";
  });

  // The master has read out outgoing data buffer
  // that buffer can be set with SPISlave.setData
  SPISlave.onDataSent([]() {
    Serial.println("Answer Sent");
  });

  // status has been received from the master.
  // The status register is a special register that bot the slave and the master can write to and read from.
  // Can be used to exchange small data or status information
  SPISlave.onStatus([](uint32_t data) {
    Serial.printf("Status: %u\n", data);
    SPISlave.setStatus(millis()); //set next status
  });

  // The master has read the status register
  SPISlave.onStatusSent([]() {
  SPISlave.setStatus(NO_STATUS);
    Serial.println("Status Sent");
  });

  // Setup SPI Slave registers and pins
  SPISlave.begin();

  // have MISO change on falling edge
  SPI1C2 |= 1 << SPIC2MISODM_S;
}

void loop() {
  server.handleClient();
}
