/* hawkless_ear.c 
   Nick Jones
   Created 4/5/19 
   Modified 4/5/19 */

#include "bat_news_travels_fast.h"

#pragma config FNOSC = PRIPLL // Oscillator selection
#pragma config POSCMOD = HS // Primary oscillator mode
#pragma config FPLLIDIV = DIV_4 // PLL input divider (20 -> 5)
#pragma config FPLLMUL = MUL_21 // PLL multiplier  ( 5x21 = 105)
#pragma config FPLLODIV = DIV_1 // PLL output divider
#pragma config FPBDIV = DIV_2 // Peripheral bus clock divider 105/2 = 52.5 Mhz
#pragma config FSOSCEN = OFF // Secondary oscillator enable
/* Clock control settings
*/
#pragma config IESO = OFF // Internal/external clock switchover
#pragma config FCKSM = CSECME // Clock switching (CSx)/Clock monitor (CMx)
#pragma config OSCIOFNC = OFF // Clock output on OSCO pin enable

/* USB Settings
*/
#pragma config UPLLEN = OFF // USB PLL enable
#pragma config UPLLIDIV = DIV_2 // USB PLL input divider
#pragma config FVBUSONIO = OFF // VBUS pin control
#pragma config FUSBIDIO = OFF // USBID pin control
#pragma config FETHIO = OFF     // Ethernet I/O
#pragma config FMIIEN = OFF     //
/* Other Peripheral Device settings
*/
#pragma config FWDTEN = OFF // Watchdog timer enable
#pragma config WDTPS = PS4096 // Watchdog timer post-scaler
#pragma config FSRSSEL = PRIORITY_7 // SRS interrupt priority
#pragma config DEBUG = ON


#pragma config	ICESEL	= ICS_PGx1	// ICE pin selection

int main() {
    initGPIO();   
    initSPI();
    initDAC();
    initADC();
    initInterrupts();
    initTimers();
    
    sdWaiting = 0;

    // set initial device mode
    deviceMode = MODE_AWAKE;
    int lastMode = MODE_AWAKE;
    
    // set the gain on the amp
    changeGain(DEFAULT_GAIN_DB);
    
    while (1) {             // loop forever
        // if interrupts changed the device mode, do something about it
        if (deviceMode != lastMode) {
            lastMode = deviceMode;
            switch (deviceMode) {
                case MODE_RECORD:
                    initSDCard();
                    startRecording();
                    break;
                case MODE_AWAKE:
                    _RB13 = 0;
                    IEC0bits.T3IE = 0;     // disable T3 (only used during recording/error states)
                    T3CONbits.ON = 0;
                    break;
                case MODE_DOWNLOAD:
                case MODE_ERROR:
                    IEC0bits.T3IE = 1;
                    PR3 = 30000000;       // set period to 500 ms
                    T3CONbits.ON = 1;
                    break;
                default:
                    break;
            }
        }
        

    }
    
    return 0;
}