/* gpio.c 
 * Nick Jones
 * Created 4/5/19
 * Modified 4/5/19 */

#include "bat_news_travels_fast.h"

/* initGPIO */
void initGPIO() {
    AD1PCFG = 0xFF;         // disable analog pins
    DDPCONbits.JTAGEN = 0;
    
    _TRISC14 = 1;           // set record button pin as input
    _TRISB13 = 0;           // set LED pin as output
    
    _RB13 = 0;              // set LED off
}

/* initADC */
void initADC() {
    AD1PCFGbits.PCFG6 = 0;          // set mic input as analog
    _TRISB6 = 1;                    // set pin (AN6) as as input
    
    AD1CHSbits.CH0SA = 0b110;       // AN6 as input to mux
    AD1CHSbits.CH0NA = 0;           // use Vref as low input
    
    AD1CON1bits.FORM = 0b000;       // set output as 16 bit int
    AD1CON1bits.SSRC = 0b010;       // sample on TMR3
    AD1CON1bits.ASAM = 1;           // sample automatically
    AD1CON2bits.VCFG = 0b111;       // use Vdd and Vss as reference voltages
    AD1CON2bits.CSCNA = 0;          // don't scan inputs
    AD1CON2bits.SMPI = 0b111;       // interrupt every 8 samples to handle buffer
    AD1CON2bits.BUFM = 1;           // use two 8 word buffers
    AD1CON2bits.ALTS = 0;           // only scan mux A
    AD1CON1bits.SIDL = 0;
}

/* startRecording */
int startRecording() {
    FIL dataFile;
    FRESULT fwrite_res;
    
    // if you can't open the file, go to error mode return false
    if (createSDFile(&dataFile) != FR_OK) {
        deviceMode = MODE_ERROR;
        return FALSE;
    }
    
    // if file is created successfully, give it time to sync
    sdWaiting = 1;            // T3 interrupt will turn this off
    IEC0bits.T3IE = 1;        // enable T3 interrupt
    PR3 = 60000;              // set period to 1 ms
    T3CONbits.ON = 1;
    while (sdWaiting == 1);
    
    // turn on the ADC
    ADCBufferCounter = 0;
    pADCBuffer = ADCBuffer1;
    AD1CON1bits.ON = 1;       
    T3CONbits.ON = 1;         // turn on sampling timer
    IEC1bits.AD1IE = 1;
    
    _RB13 = 1;                // turn on LED
    
    uint32 numread = 0;
    
    // continue until recording stops
    while (deviceMode == MODE_RECORD) {
        // wait till the buffer is full or recording stops
        while (ADCBufferCounter < SD_WRITE_SIZE && deviceMode == MODE_RECORD);

        // write buffer to SD card
        if (ADCBufferCounter > 0) {
            fwrite_res = f_write(&dataFile, pADCBuffer, ADCBufferCounter, &numread);
            f_sync(&dataFile);
            pADCBuffer = ADCBuffer2;
            ADCBufferCounter = 0;
        }
    
        // repeat for second buffer
        while (ADCBufferCounter < SD_WRITE_SIZE && deviceMode == MODE_RECORD);

        if (ADCBufferCounter > 0) {
            fwrite_res = f_write(&dataFile, pADCBuffer, ADCBufferCounter, &numread);
            f_sync(&dataFile);
            pADCBuffer = ADCBuffer1;
            ADCBufferCounter = 0;
        }
    }
    
    // after recording is done, disable everything
    _RB13 = 0;                  // turn LED off
    IEC1bits.AD1IE = 0;
    AD1CON1bits.ON = 0;       // turn off ADC
    T3CONbits.ON = 0;         // turn off timer
    
    // give it time to sync again
    sdWaiting = 1;            // T3 will turn this off
    IEC0bits.T3IE = 1;        // enable T3 interrupt
    PR3 = 30000000;              // set period to 500 ms
    T3CONbits.ON = 1;
    while (sdWaiting == 1);
    
    f_close(&dataFile);          // close the file
    uninitSDCard();
    
    return TRUE;
}